package com.example.zcatalyst_sdk



object IAMSDKConstants {

    object MethodChannel {
        const val DOMAIN = "catalyst.zoho.com"
        const val NAME = "iam_sdk"
    }

    object MethodCall{
        const val INIT = "init";
        const val INIT_WITH_CONFIGS = "initWithSDKConfigs";
        const val LOGIN = "login";
        const val SIGN_UP = "signUp";
        const val GET_ACCESS_TOKEN = "getAccessToken";
        const val IS_USER_LOGGED_IN = "isUserLoggedIn";
        const val LOGOUT = "logout";
        const val GET_JWT_CLIENT_DETAILS = "get_jwt_client_details"
        const val GET_OAUTH_TOKEN_WITH_JWT_TOKEN = "get_oauth_token_with_jwt_token"
    }

    object MethodArguments{
        const val ACCOUNTS_PORTAL_URL = "accountsPortalBaseURL"
        const val REDIRECT_URL_SCHEME = "redirectURLScheme"
        const val CLIENT_ID = "clientID"
        const val CLIENT_SECRET = "clientSecret"
        const val PORTAL_ID = "portalID"
        const val SCOPES = "oauthScopes"
        const val ENVIRONMENT = "environment"
        const val API_BASE_URL = "apiBaseURL"
        const val PROJECT_ID = "projectID"
        const val PRINT_STACK_TRACE = "printStackTrace"
        const val IS_CUSTOM_LOGIN = "isCustomLogin"
        const val CUSTOM_PARAMS = "customParams"
        const val URL = "url"
        const val JWT_TOKEN = "jwtToken"

    }

    object SDKConfigKeys{
        const val REDIRECT_URL_SCHEME = "redirect_url_scheme"
        const val CLIENT_ID = "client_id"
        const val CLIENT_SECRET = "client_secret"
        const val PORTAL_ID = "portal_id"
        const val API_BASE_URL = "api_base_url"
        const val PROJECT_ID = "project_id"
        const val PRINT_STACK_TRACE = "print_stacktrace"
        const val REQUEST_HEADERS = "requestHeaders"
        const val JWT_CLIENT_ID = "jwt_client_id"
        const val JWT_CLIENT_SECRET = "jwt_client_secret"
        const val ACCOUNTS_PORTAL_DOMAIN = "accounts_portal_domain"
        const val SCOPES = "scopes"
    }

    object ErrorCodes{
        const val INTERNAL_ERROR = "INTERNAL_ERROR"
        const val INITIALIZATION_ERROR = "INITIALIZATION_ERROR"
        const val ARGUMENT_MISSING = "ARGUMENT_MISSING"
        const val TOKEN_FETCH_ERROR = "TOKEN_FETCH_ERROR"
        const val LOGIN_ERROR = "LOGIN_ERROR"
        const val LOGOUT_FAILED = "LOGOUT_FAILED"
        const val CUSTOM_LOGIN_DISABLED = "CUSTOM_LOGIN_IS_DISABLED"
    }

    object ErrorMessage{
        const val INITIALIZATION_ERROR = "Zoho Catalyst SDK must be initialized for this operation."
        const val MANDATORY_CONFIG_MISSING = "Please ensure all mandatory properties are given values"
        const val ACCOUNTS_PORTAL_URL = "SDK initialization failed! The required argument '${MethodArguments.ACCOUNTS_PORTAL_URL}' is null."
        const val CLIENT_ID = "SDK initialization failed! The required argument '${MethodArguments.CLIENT_ID}' is null."
        const val CLIENT_SECRET = "SDK initialization failed! The required argument '${MethodArguments.CLIENT_SECRET}' is null."
        const val REDIRECT_URL_SCHEME = "SDK initialization failed! The required argument '${MethodArguments.REDIRECT_URL_SCHEME}' is null."
        const val PORTAL_ID = "SDK initialization failed! The required argument '${MethodArguments.REDIRECT_URL_SCHEME}' is null."
        const val SCOPES = "SDK initialization failed! The required argument '${MethodArguments.SCOPES}' is null."
        const val ENVIRONMENT = "SDK initialization failed! The required argument '${MethodArguments.ENVIRONMENT}' is null."
        const val IS_CUSTOM_LOGIN = "SDK initialization failed! The required argument '${MethodArguments.IS_CUSTOM_LOGIN}' is null."
        const val URL = "SDK initialization failed! The required argument '${MethodArguments.URL}' is null."
        const val LOGOUT_FAILED = "PLEASE TRY AGAIN LATER."
        const val JWT_TOKEN = "The required argument '${MethodArguments.JWT_TOKEN}' is null."
        const val CUSTOM_LOGIN_DISABLED = "The parameters required for Custom Login could not be found, make sure you have enabled Custom Login for the project and re-download the property file."

    }
    const val CONFIG_PROPERTY_NOT_AVAILABLE = "NA"
    const val LOG_TAG = "ZCatalyst_SDK_LOG ::: "
}

enum class ZCatalystEnvironment {
    DEVELOPMENT, PRODUCTION;

    override fun toString(): String {

        return when (this) {
            DEVELOPMENT -> "DEVELOPMENT"
            PRODUCTION -> "PRODUCTION"
        }
    }

    companion object {
        fun of(value: String): ZCatalystEnvironment{
            return when (value) {
                "DEVELOPMENT" -> DEVELOPMENT
                "PRODUCTION" -> PRODUCTION
                else -> DEVELOPMENT
            }
        }
    }
}