//
//  IAMClientSDKPlugin.swift
//  zcatalyst_sdk
//
//  Created by vignesh-pt5452 on 16/12/22.
//

import Foundation


struct IAMSDKConstants {

    struct MethodChannel {

        static let domain = "catalyst.zoho.com"
        static let name = "iam_sdk"

    }

    struct MethodCall{

        static let initWithPlistFile = "init";
        static let initWithConfigs = "initWithSDKConfigs";
        static let login = "login";
        static let signUp = "signUp";
        static let getAccessToken = "getAccessToken";
        static let isUserLoggedIn = "isUserLoggedIn";
        static let logout = "logout";
        static let getOauthTokenWithJwtToken = "get_oauth_token_with_jwt_token";

    }

    struct MethodArguments{

        static let accountsPortalUrl = "accountsPortalBaseURL"
        static let redirectUrlScheme = "redirectURLScheme"
        static let clientId = "clientID"
        static let clientSecret = "clientSecret"
        static let portalId = "portalID"
        static let scopes = "oauthScopes"
        static let environment = "environment"
        static let apiBaseUrl = "apiBaseURL"
        static let projectId = "projectID"
        static let readTimeOutInSec = "readTimeOutInSec"
        static let writeTimeoutInSec = "writeTimeOutInSec"
        static let connectTimeOutInSec = "connectTimeOutInSec"
        static let printStackTrace = "printStackTrace"
        static let isCustomLogin = "isCustomLogin"
        static let customParams = "customParams"
        static let url = "url"
        static let jwtToken = "jwtToken"


    }

    struct SDKConfigKeys{

        static let redirectUrlScheme = "redirect_url_scheme"
        static let clientId = "client_id"
        static let clientSecret = "client_secret"
        static let portalId = "portal_id"
        static let apiBaseUrl = "api_base_url"
        static let projectId = "project_id"
        static let readTimeOutInSec = "read_timeout_in_sec"
        static let writeTimeoutInSec = "write_timeout_in_sec"
        static let connectTimeOutInSec = "connect_timeout_in_sec"
        static let printStackTrace = "print_stacktrace"
        static let requestHeaders = "requestHeaders"
        static let jwtClientId = "jwt_client_id"
        static let jwtClientSecret = "jwt_client_secret"
        static let scopes = "scopes"
        static let accountsPortalDomain = "accounts_portal_domain"

    }

    struct ErrorCodes{

        static let internalError = "INTERNAL_ERROR"
        static let initializationError = "INITIALIZATION_ERROR"
        static let argumentMissing = "ARGUMENT_MISSING"
        static let tokenFetchError = "TOKEN_FETCH_ERROR"
        static let loginFailed = "LOGIN_FAILED"
        static let logoutFailed = "LOGOUT_FAILED"
        static let fileNotFound = "FILE_NOT_FOUND"
        static let customLoginDisabled = "CUSTOM_LOGIN_IS_DISABLED"

    }

    struct ErrorMessage{

        static let initializationError = "Zoho Catalyst SDK must be initialized for this operation."
        static let missingDartArguments = "The Dart arguments map is missing in the method call: "
        static let logoutFailed = "PLEASE TRY AGAIN LATER."
        static let fileNotFound = "Configuration failed, check if you have added AppConfiguration.plist"
        static let customLoginDisabled = "The parameters required for Custom Login could not be found, make sure you have enabled Custom Login for the project and re-download the property file."
    }

    static let configPropertyNotAvailable = "NA"

}

enum ZCatalystEnvironment: String{

    case DEVELOPMENT
    case PRODUCTION

    static func of(_ environment: String) -> ZCatalystEnvironment {
        switch(environment){
        case "DEVELOPMENT":
            return ZCatalystEnvironment.DEVELOPMENT
        case "PRODUCTION":
            return ZCatalystEnvironment.PRODUCTION
        default:
            return ZCatalystEnvironment.DEVELOPMENT
        }
    }

}

enum ZCatalystError : Error{
    case fileNotFound
    case processingError(code: String, message: String, details: Any?)
}

extension Dictionary{
    func optValue(key: Key) -> Any?
    {
        if( self.hasValue( forKey : key ) ), let value = self[ key ]
        {
            return value
        }
        else
        {
            return nil
        }
    }

    func hasValue(forKey : Key) -> Bool
    {
        return self[forKey] != nil && !(self[forKey] is NSNull)
    }

    func getValue( key : Key ) throws -> Any
    {
        guard let value = optValue( key: key ) else
        {
            throw ZCatalystError.processingError( code : IAMSDKConstants.ErrorCodes.internalError, message : "\( key ) - Key not found - \( key )", details : nil )
        }
        return value
    }
    func getValue< T >( key : Key ) throws -> T
    {
        guard let value = try getValue( key: key ) as? T else
        {

            throw ZCatalystError.processingError( code : IAMSDKConstants.ErrorCodes.internalError, message : "\( key ) - Expected type -> \( T.self )", details : nil )
        }
        return value
    }
}
