part of '../api_request.dart';

class ColumnAPIHandler {
  final String tableIdentifier;

  ColumnAPIHandler({required this.tableIdentifier});

  Future<void> getColumns(
      {required Function(APIResponse response, List<ZCatalystColumn> columns)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.COLUMN}',
      method: HttpMethod.GET,
    )._sendRequest(
      onSuccess: (APIResponse response) {
        var responseData = response.responseJson[APIConstant.DATA];
        List<ZCatalystColumn> columns = columnFromJson(responseData, onFailed);
        return onSuccess(response, columns);
      },
      onFailed: (ZCatalystException exception) {
        return onFailed(exception);
      },
    );
  }

  Future<void> getColumn(
      {required String identifier,
      required Function(APIResponse response, ZCatalystColumn column) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
      endPoint:
          '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.COLUMN}/$identifier',
      method: HttpMethod.GET,
    )._sendRequest(
      onSuccess: (APIResponse response) {
        var responseData = response.responseJson[APIConstant.DATA];
        ZCatalystColumn column = fromJson(responseData, onFailed);
        return onSuccess(response, column);
      },
      onFailed: (ZCatalystException exception) {
        return onFailed(exception);
      },
    );
  }

  List<ZCatalystColumn> columnFromJson(List<dynamic> data,
          Function(ZCatalystException exception) onFailed) =>
      List<ZCatalystColumn>.from(
          data.map<ZCatalystColumn>((x) => fromJson(x, onFailed)));

  ZCatalystColumn fromJson(Map<String, dynamic> json,
      Function(ZCatalystException exception) onFailed) {
    if (json["column_id"] == null) {
      return onFailed(ZCatalystSDKException(message: 'column id is null'));
    }
    return ZCatalystColumn(
      sequence: json["column_sequence"],
      name: json["column_name"],
      category: json["category"],
      dataType: json["data_type"],
      maxLength: json["max_length"],
      isMandatory: json["is_mandatory"],
      decimalDigits: json["decimal_digits"],
      isUnique: json["is_unique"],
      isSearchIndexEnabled: json["search_index_enabled"],
      id: json["column_id"],
      defaultValue: json["default_value"],
      constraintType: json["constraint_type"],
    );
  }
}
