part of '../api_request.dart';

class FileAPIHandler {
  int folderId;

  FileAPIHandler({required this.folderId});

  Future<void> getFile(
      int id,
      Function(APIResponse response, ZCatalystFile file) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await APIRequest._(
      endPoint: 'folder/$folderId/file/$id',
      method: HttpMethod.GET,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      ZCatalystFile file = fromJson(responseData, onFailed);
      onSuccess(response, file);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> getFiles(
      Function(APIResponse response, List<ZCatalystFile> file) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await APIRequest._(
      endPoint: 'folder/$folderId/file',
      method: HttpMethod.GET,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      List<ZCatalystFile> files = fileFromJson(responseData, onFailed);
      onSuccess(response, files);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> uploadFile(
      String filePath,
      String fileName,
      Function(APIResponse response, ZCatalystFile file) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    Map<String, String> requestBody = {'file_name': fileName};
    File file = File(filePath);
    try {
      FileUtil.validateFileSize(file, APIConstant.MAX_ALLOWED_FILE_SIZE_IN_MB);
    } on ZCatalystException catch (ex) {
      return onFailed(ex);
    }
    await APIRequest._(
      endPoint: 'folder/$folderId/file',
      method: HttpMethod.POST,
    )._uploadFile(
        filePath: filePath,
        body: requestBody,
        onSuccess: (APIResponse response) {
          var responseData = response.responseJson[APIConstant.DATA];
          ZCatalystFile file = fromJson(responseData, onFailed);
          onSuccess(response, file);
        },
        onFailed: (ZCatalystException exception) {
          onFailed(exception);
        });
  }

  Future<void> downloadFile(int id,
      {required String path,
      required String name,
      required Function(FileAPIResponse response, String filePath) onSuccess,
      required Function(int bytesWritten, int contentLength, double percentage)
          onProgress,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
            endPoint: 'folder/$folderId/file/$id/download',
            method: HttpMethod.GET)
        ._downloadFile(
            filePath: path,
            fileName: name,
            onSuccess: onSuccess,
            onProgress: onProgress,
            onFailed: onFailed);
  }

  Future<void> downloadFileAsStream(int id,
      {required Function(FileAPIResponse response, http.ByteStream fileStream)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
            endPoint: 'folder/$folderId/file/$id/download',
            method: HttpMethod.GET)
        ._downloadFileAsStream(onSuccess: onSuccess, onFailed: onFailed);
  }

  Future<void> deleteFile(int id, Function(APIResponse response) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await APIRequest._(
      endPoint: 'folder/$folderId/file/$id/download',
      method: HttpMethod.DELETE,
    )._sendRequest(
      onSuccess: (APIResponse response) {
        onSuccess(response);
      },
      onFailed: (ZCatalystException exception) {
        onFailed(exception);
      },
    );
  }

  List<ZCatalystFile> fileFromJson(List<dynamic> data,
          Function(ZCatalystException exception) onFailed) =>
      List<ZCatalystFile>.from(
          data.map<ZCatalystFile>((x) => fromJson(x, onFailed)));

  ZCatalystFile fromJson(Map<String, dynamic> json,
      Function(ZCatalystException exception) onFailed) {
    if (json['id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'File ID must not be null'));
    } else if (json['created_by']['last_name'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Last Name must not be null'));
    } else if (json['created_by']['email_id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Email ID must not be null'));
    } else if (json['modified_by']['last_name'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Last Name must not be null'));
    } else if (json['modified_by']['email_id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Email ID must not be null'));
    }
    var createdByUserDetails = json['created_by'] as Map<String, dynamic>;
    ZCatalystUserDelegate createdBy = ZCatalystUserDelegate(
        createdByUserDetails['last_name'], createdByUserDetails['email_id']);
    var modifiedByUserDetails = json['modified_by'] as Map<String, dynamic>;
    ZCatalystUserDelegate modifiedBy = ZCatalystUserDelegate(
        modifiedByUserDetails['last_name'], modifiedByUserDetails['email_id']);
    ZCatalystFile file = ZCatalystFile(
        folderId: folderId,
        id: json['id'],
        name: json['file_name'],
        size: json['file_size'],
        createdTime: json['created_time'],
        createdBy: createdBy,
        modifiedTime: json['modified_time'],
        modifiedBy: modifiedBy);
    return file;
  }
}
