part of '../api_request.dart';

class FolderAPIHandler {
  Future<void> getFolder(
      int id,
      Function(APIResponse response, ZCatalystFolder folder) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await APIRequest._(
      endPoint: 'folder/$id',
      method: HttpMethod.GET,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      ZCatalystFolder folder = fromJson(responseData, onFailed);
      onSuccess(response, folder);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> getFolders(
      Function(APIResponse response, List<ZCatalystFolder> folders) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await APIRequest._(
      endPoint: 'folder',
      method: HttpMethod.GET,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      List<ZCatalystFolder> folders = folderFromJson(responseData, onFailed);
      onSuccess(response, folders);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  List<ZCatalystFolder> folderFromJson(List<dynamic> data,
          Function(ZCatalystException exception) onFailed) =>
      List<ZCatalystFolder>.from(
          data.map<ZCatalystFolder>((x) => fromJson(x, onFailed)));

  ZCatalystFolder fromJson(Map<String, dynamic> json,
      Function(ZCatalystException exception) onFailed) {
    if (json['id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Folder ID must not be null'));
    } else if (json['created_by']['last_name'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Last Name must not be null'));
    } else if (json['created_by']['email_id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Email ID must not be null'));
    } else if (json['modified_by']['last_name'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Last Name must not be null'));
    } else if (json['modified_by']['email_id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Email ID must not be null'));
    }
    ZCatalystFolder folder = ZCatalystFolder(json['id']);
    var createdByUserDetails = json['created_by'] as Map<String, dynamic>;
    ZCatalystUserDelegate createdBy = ZCatalystUserDelegate(
        createdByUserDetails['last_name'], createdByUserDetails['email_id']);
    var modifiedByUserDetails = json['modified_by'] as Map<String, dynamic>;
    ZCatalystUserDelegate modifiedBy = ZCatalystUserDelegate(
        modifiedByUserDetails['last_name'], modifiedByUserDetails['email_id']);
    folder.name = json['folder_name'];
    folder.createdTime = json['created_time'];
    folder.createdBy = createdBy;
    folder.modifiedTime = json['modified_time'];
    folder.modifiedBy = modifiedBy;
    return folder;
  }
}
