part of '../api_request.dart';

class SearchAPIhandler {
  ZCatalystSearchOptions searchOptions;

  SearchAPIhandler(this.searchOptions);

  Future<void> search(
      Function(APIResponse response, Map<String, dynamic> result) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    Map<String, dynamic> body = toJson(searchOptions);
    await APIRequest._(
      endPoint: 'search',
      method: HttpMethod.POST,
      body: body,
    )._sendRequest(
      onSuccess: (APIResponse response) {
        var responseData = response.responseJson[APIConstant.DATA];
        onSuccess(response, responseData);
      },
      onFailed: (ZCatalystException exception) {
        onFailed(exception);
      },
    );
  }

  Map<String, dynamic> toJson(ZCatalystSearchOptions src) {
    Map<String, dynamic> searchJson = <String, dynamic>{};
    searchJson['search'] = src._searchString;
    if (src.startIndex != null) searchJson['start'] = src.startIndex;
    if (src.endIndex != null) searchJson['end'] = src.endIndex;

    Map<String, dynamic> searchTableColumnsJson = <String, dynamic>{};
    if (src.searchColumns.isNotEmpty) {
      var searchColumns = src.searchColumns;
      for (var searchColumn in searchColumns) {
        var tableName = searchColumn.first;
        var columns = searchColumn.second;
        List<String> columnsArray = [];
        for (var column in columns) {
          columnsArray.add(column);
        }
        searchTableColumnsJson[tableName] = columnsArray;
      }
      searchJson['search_table_columns'] = searchTableColumnsJson;
    }

    Map<String, dynamic> displayTableColumnsJson = <String, dynamic>{};
    if (src.displayColumns.isNotEmpty) {
      var displayColumns = src.displayColumns;
      for (var displayColumn in displayColumns) {
        var tableName = displayColumn.first;
        var columns = displayColumn.second;
        List<String> columnsArray = [];
        for (var column in columns) {
          columnsArray.add(column);
        }
        displayTableColumnsJson[tableName] = columnsArray;
      }
      searchJson['select_table_columns'] = displayTableColumnsJson;
    }

    Map<String, dynamic> sortColumnsJson = <String, dynamic>{};
    if (src.sortColumns.isNotEmpty) {
      var sortColumns = src.sortColumns;
      for (var sortColumn in sortColumns) {
        var tableName = sortColumn.first;
        var columnName = sortColumn.second;
        sortColumnsJson[tableName] = columnName;
      }
      searchJson['order_by'] = sortColumnsJson;
    }
    return searchJson;
  }
}
