part of '../api_request.dart';

class UserAPIHandler {
  Future<void> getCurrentUser(
      {required Function(APIResponse response, ZCatalystUser user) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
      endPoint: '${Endpoints.USER}/current',
      method: HttpMethod.GET,
    )._sendRequest(onSuccess: ((APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      ZCatalystUser userObject = fromJson(responseData, onFailed);
      onSuccess(response, userObject);
    }), onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> signUp(
      {required ZCatalystUserDelegate newUser,
      required Function(APIResponse response, ZCatalystUser user) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    Map<String, dynamic> body = {
      "zaid": ZCatalystApp.configs.portalID,
      "user_details": {
        "last_name": newUser.lastName,
        "email_id": newUser.emailId,
      },
      "platform_type": Platform.isAndroid ? "android" : "ios"
    };
    await APIRequest._(
      endPoint: 'project-user/signup',
      method: HttpMethod.POST,
      body: body,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData =
          response.responseJson[APIConstant.DATA][APIConstant.USER_DETAILS];
      ZCatalystUser user = fromJson(responseData, onFailed);
      onSuccess(response, user);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  ZCatalystUser fromJson(Map<String, dynamic> json,
      Function(ZCatalystException exception) onFailed) {
    if (json['user_id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'User ID must not be null'));
    } else if (json['last_name'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Last Name must not be null'));
    } else if (json['email_id'] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Email ID must not be null'));
    } else if (json['role_details'] != null) {
      var roleDetails = json['role_details'];
      if (roleDetails['role_id'] == null) {
        return onFailed(
            ZCatalystSDKException(message: 'Role ID must not be null'));
      }
      if (roleDetails['role_name'] == null) {
        return onFailed(
            ZCatalystSDKException(message: 'Role Name must not be null'));
      }
    }
    ZCatalystRole roleDetailsObj = ZCatalystRole(
      id: json["role_details"]["role_id"],
      name: json["role_details"]["role_name"],
    );
    return ZCatalystUser(
        emailId: json["email_id"],
        lastName: json["last_name"],
        zuid: json["zuid"],
        zaaid: json["zaaid"],
        firstName: json["first_name"],
        isConfirmed: json["is_confirmed"],
        status: json["status"],
        id: json["user_id"],
        userType: json["user_type"],
        createdTime: json["created_time"],
        modifiedTime: json["modified_time"],
        invitedTime: json["invited_time"],
        roleDetails: roleDetailsObj);
  }
}
