import '../api_request.dart';
import 'dart:convert';
import 'package:http/http.dart';
import '../../util/api_constants.dart';
import '../../util/pair.dart';
import '../../exception/zcatalyst_exception.dart';
import '../../exception/zcatalyst_sdk_exception.dart';

part 'api_response.dart';
part 'file_api_response.dart';

class BaseAPIResponse {
  BaseAPIResponse(BaseResponse response) : statusCode = response.statusCode {
    var contentType = response.headers[APIConstant.CONTENT_TYPE];
    if (contentType == null || contentType.contains('text/html')) {
      throw ZCatalystSDKException(message: ErrorMessage.HTML_PAGE_AS_REPONSE);
    }
    _setResponseHeaders(response);
    _setResponseJSON();
    _processResponse();
  }

  int statusCode;

  late ResponseHeaders responseHeaders;

  dynamic responseJson;

  void _setResponseHeaders(BaseResponse response) {
    responseHeaders = ResponseHeaders(response);
  }

  void _setResponseJSON() {
    return;
  }

  void _processResponse() {
    return;
  }

  void _handleFaultyResponses(Map<String, dynamic> errorJson) {
    try {
      Pair<String, String> errorDetails = _getErrorDetails(errorJson);
      String errorCode = errorDetails.first;
      String errorMessage = errorDetails.second;
      throw ZCatalystException(code: errorCode, errorMessage: errorMessage);
    } on FormatException catch (ex) {
      throw ZCatalystSDKException(originalException: ex);
    }
  }

  Pair<String, String> _getErrorDetails(Map<String, dynamic> errorJson) {
    String errorCode = ErrorCode.INVALID_DATA;
    String errorMessage = ErrorMessage.INVALID_DATA;
    if (errorJson.containsKey(APIConstant.ERROR)) {
      errorCode = ErrorCode.FUNCTION_EXECUTION_ERROR;
      errorMessage = errorJson[APIConstant.ERROR];
    } else if (errorJson.containsKey(APIConstant.OUTPUT)) {
      errorCode = ErrorCode.FUNCTION_EXECUTION_ERROR;
      errorMessage = errorJson[APIConstant.OUTPUT];
    } else if (errorJson.containsKey(APIConstant.DATA)) {
      Map<String, dynamic> errorData = errorJson[APIConstant.DATA];
      if (errorData.containsKey(APIConstant.ERROR_CODE)) {
        errorCode = errorData[APIConstant.ERROR_CODE];
      }
      if (errorData.containsKey(APIConstant.MESSAGE)) {
        errorMessage = errorData[APIConstant.MESSAGE];
      }
    }
    Pair<String, String> errorDetails = Pair(errorCode, errorMessage);
    return errorDetails;
  }
}

class ResponseHeaders {
  String? dateTime;

  Map<String, dynamic> headerVsValue = <String, dynamic>{};

  ResponseHeaders(dynamic response) {
    var responseHeaders = response.headers;
    var date = responseHeaders[APIConstant.DATE];
    if (date != null) {
      dateTime = date;
    }
    for (var key in responseHeaders.keys) {
      if (responseHeaders[key] != null) {
        headerVsValue[key] = responseHeaders[key];
      }
    }
  }
}
