part of '../setup/zcatalyst_app.dart';

class IAMClientSDKPlugin {
  static MethodChannel _methodChannel = const MethodChannel(
      '${MethodChannelConstants.CHANNEL_DOMAIN}/${MethodChannelConstants.CHANNEL}');

  static Future<Map<String, dynamic>> _init(
      ZCatalystEnvironment environment, bool isCustomLogin) async {
    Map<String, dynamic> arguments = {
      MethodChannelArguments.ENVIRONMENT: environment.toString(),
      MethodChannelArguments.IS_CUSTOM_LOGIN: isCustomLogin
    };
    try {
      Map<String, dynamic>? sdkConfigsJson =
          await _methodChannel.invokeMapMethod<String, dynamic>(
              MethodChannelConstants.METHOD_CALL_INIT, arguments);
      return sdkConfigsJson!;
    } on PlatformException catch (ex) {
      if (ex.code == ErrorCode.INTERNAL_ERROR) {
        throw ZCatalystSDKException(message: ex.message);
      } else {
        throw ZCatalystException(code: ex.code, errorMessage: ex.message);
      }
    } on MissingPluginException catch (ex) {
      throw ZCatalystSDKException(originalException: ex);
    }
  }

  static Future<void> _initWithSDKConfigs(
      ZCatalystSDKConfigs sdkConfigs) async {
    Map<String, dynamic> arguments = {
      MethodChannelArguments.ACCOUNTS_PORTAL_URL:
          sdkConfigs.accountsPortalBaseURL,
      MethodChannelArguments.REDIRECT_URL_SCHEME: sdkConfigs.redirectURLScheme,
      MethodChannelArguments.CLIENT_ID:
          sdkConfigs.clientID != APIConstant.STRING_MOCK
              ? sdkConfigs.clientID
              : sdkConfigs.jwtClientID,
      MethodChannelArguments.CLIENT_SECRET:
          sdkConfigs.clientSecret != APIConstant.STRING_MOCK
              ? sdkConfigs.clientSecret
              : sdkConfigs.jwtClientSecret,
      MethodChannelArguments.PORTAL_ID: sdkConfigs.portalID,
      MethodChannelArguments.SCOPES: sdkConfigs.oauthScopes
    };
    try {
      await _methodChannel.invokeMethod(
          MethodChannelConstants.METHOD_CALL_INIT_WITH_CONFIGS, arguments);
    } on PlatformException catch (ex) {
      throw ZCatalystSDKException(
          message: ex.message, errorDetails: ex.details);
    } on MissingPluginException catch (ex) {
      throw ZCatalystSDKException(message: ex.message, originalException: ex);
    }
  }

  static Future<void> _login(
      {Map<String, String>? customParams,
      required Function() onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      await _methodChannel.invokeMethod(
          MethodChannelConstants.METHOD_CALL_LOGIN, <String, dynamic>{
        MethodChannelArguments.CUSTOM_PARAMS: customParams,
      });
      onSuccess();
    } on PlatformException catch (ex) {
      if (ex.code == ErrorCode.INTERNAL_ERROR) {
        onFailed(ZCatalystSDKException(message: ex.message));
      } else {
        onFailed(ZCatalystException(code: ex.code, errorMessage: ex.message));
      }
    } on MissingPluginException catch (ex) {
      onFailed(ZCatalystSDKException(originalException: ex));
    }
  }

  static Future<bool> _isUserLoggedIn() async {
    return await _methodChannel
        .invokeMethod(MethodChannelConstants.METHOD_CALL_IS_USER_LOGGED_IN);
  }

  static Future<String> _getAccessToken() async {
    try {
      return await _methodChannel
          .invokeMethod(MethodChannelConstants.METHOD_CALL_GET_ACCESS_TOKEN);
    } on PlatformException catch (ex) {
      if (ex.code == ErrorCode.INTERNAL_ERROR) {
        throw ZCatalystSDKException(message: ex.message);
      } else {
        throw ZCatalystException(code: ex.code, errorMessage: ex.message);
      }
    } on MissingPluginException catch (ex) {
      throw ZCatalystSDKException(originalException: ex);
    }
  }

  static Future<void> _getOauthToken({required String jwtToken}) async {
    try {
      await _methodChannel.invokeMethod(
          MethodChannelConstants.METHOD_CALL_GET_OAUTH_TOKEN_WITH_JWT_TOKEN, {
        MethodChannelArguments.JWT_TOKEN: jwtToken,
      });
    } on PlatformException catch (ex) {
      if (ex.code == ErrorCode.INTERNAL_ERROR) {
        throw ZCatalystSDKException(message: ex.message);
      } else {
        throw ZCatalystException(code: ex.code, errorMessage: ex.message);
      }
    } on MissingPluginException catch (ex) {
      throw ZCatalystSDKException(originalException: ex);
    }
  }

  static Future<void> _logout(
      {required Function() onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      await _methodChannel
          .invokeMethod(MethodChannelConstants.METHOD_CALL_LOGOUT);
      onSuccess();
    } on PlatformException catch (ex) {
      if (ex.code == ErrorCode.INTERNAL_ERROR) {
        onFailed(ZCatalystSDKException(message: ex.message));
      } else {
        onFailed(ZCatalystException(code: ex.code, errorMessage: ex.message));
      }
    } on MissingPluginException catch (ex) {
      onFailed(ZCatalystSDKException(originalException: ex));
    }
  }
}
