part of '../api/api_request.dart';

class ZCatalystTable {
  String identifier;

  ZCatalystTable(this.identifier);

  Future<void> createRows(
      {required List<ZCatalystRow> rows,
      required Function(APIResponse response, List<ZCatalystRow> rows)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await RowAPIHandler(tableIdentifier: identifier)
        .createRows(rows: rows, onSuccess: onSuccess, onFailed: onFailed);
  }

  Future<void> updateRows(
      {required List<ZCatalystRow> rows,
      required Function(APIResponse response, List<ZCatalystRow> rows)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await RowAPIHandler(tableIdentifier: identifier)
        .updateRows(rows: rows, onSuccess: onSuccess, onFailed: onFailed);
  }

  Future<void> getRow(
      {required int id,
      required Function(APIResponse response, ZCatalystRow row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await RowAPIHandler(tableIdentifier: identifier)
        .getRow(id: id, onSuccess: onSuccess, onFailed: onFailed);
  }

  Future<void> getRows(
      {int? maxRows,
      String? nextToken,
      required Function(
              APIResponse response, List<ZCatalystRow> rows, ResponseInfo info)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await RowAPIHandler(tableIdentifier: identifier).getRows(
        maxRows: maxRows,
        nextToken: nextToken,
        onSuccess: onSuccess,
        onFailed: onFailed);
  }

  Future<void> deleteRow(
      {required int id,
      required Function(APIResponse response) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await RowAPIHandler(tableIdentifier: identifier)
        .deleteRow(id: id, onSuccess: onSuccess, onFailed: onFailed);
  }

  Future<void> getColumns(
      {required Function(APIResponse response, List<ZCatalystColumn> columns)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await ColumnAPIHandler(tableIdentifier: identifier)
        .getColumns(onSuccess: onSuccess, onFailed: onFailed);
  }

  Future<void> getColumn(
      {required String identifier,
      required Function(APIResponse response, ZCatalystColumn column) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    ColumnAPIHandler(tableIdentifier: this.identifier).getColumn(
        identifier: identifier, onSuccess: onSuccess, onFailed: onFailed);
  }

  ZCatalystRow newRow() {
    ZCatalystRow newRow = ZCatalystRow(identifier);
    newRow._isCreate = true;
    return newRow;
  }
}

class ResponseInfo {
  bool moreRecords;
  String? nextToken;

  ResponseInfo(this.moreRecords, this.nextToken);
}
