import '../api/api_request.dart';
import '../api/response/base_api_response.dart';
import '../exception/zcatalyst_exception.dart';
import 'zcatalyst_file.dart';
import '../crud/zcatalyst_user_delegate.dart';

class ZCatalystFolder {
  ///The unique **id** of the folder.
  int id;

  ///The name of the folder.
  String? name;

  ///The time in Date-Time format at which the current file is created.
  String? createdTime;

  ///The details of the user who created this folder.
  ZCatalystUserDelegate? createdBy;

  ///TThe time in Date-Time format at which the current file is modified.
  String? modifiedTime;

  ///The details of the user who recently modified this folder.
  ZCatalystUserDelegate? modifiedBy;

  ZCatalystFolder(this.id);

  ///Fetches the details of a specific file with the [id].
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] object & the [ZCatalystFile] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> getFile(
      int id,
      Function(APIResponse response, ZCatalystFile file) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await FileAPIHandler(folderId: this.id).getFile(id, onSuccess, onFailed);
  }

  ///Fetches the details of all the files present in the current folder.
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] object & a [List] of [ZCatalystFile] objects.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> getFiles(
      Function(APIResponse response, List<ZCatalystFile> file) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await FileAPIHandler(folderId: id).getFiles(onSuccess, onFailed);
  }

  ///Uploads a new file in the current folder.
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] & [ZCatalystFile] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> uploadFile(
      String filePath,
      String fileName,
      Function(APIResponse response, ZCatalystFile file) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await FileAPIHandler(folderId: id)
        .uploadFile(filePath, fileName, onSuccess, onFailed);
  }
}
