part of '../api/api_request.dart';

enum SearchPattern {
  EQUALS,
  STARTS_WITH;

  String getSearchString(String text) {
    switch (this) {
      case SearchPattern.EQUALS:
        return text;
      case SearchPattern.STARTS_WITH:
        return '$text*';
    }
  }
}

class ZCatalystSearchOptions {
  String searchText;

  List<TableColumn> searchTableColumns;

  String _searchString;

  int? startIndex;

  int? endIndex;

  final List<Pair<String, List<String>>> _searchColumns = [];

  final List<Pair<String, List<String>>> _displayColumns = [];

  final List<Pair<String, String>> _sortColumns = [];

  List<Pair<String, List<String>>> get searchColumns => _searchColumns;

  List<Pair<String, List<String>>> get displayColumns => _displayColumns;

  List<Pair<String, String>> get sortColumns => _sortColumns;

  ZCatalystSearchOptions(this.searchText, this.searchTableColumns)
      : _searchString = SearchPattern.EQUALS.getSearchString(searchText) {
    setSearchColumns(searchTableColumns);
  }

  void setSearchPattern(SearchPattern pattern) {
    _searchString = pattern.getSearchString(searchText);
  }

  void setSearchColumns(List<TableColumn> searchTableColumns) {
    for (var searchTable in searchTableColumns) {
      _searchColumns.add(searchTable.getTableNameVsColumns());
    }
  }

  void setDisplayColumns(List<TableColumn> displayTableColumns) {
    for (var displayTable in displayTableColumns) {
      _displayColumns.add(displayTable.getTableNameVsColumns());
    }
  }

  void addSortColumn(String tableName, String columnName) {
    _sortColumns.add(Pair(tableName, columnName));
  }
}

class TableColumn {
  String tableName;

  List<String> columns = [];

  TableColumn(this.tableName);

  void addColumn(String columnName) {
    columns.add(columnName);
  }

  void addColumns(List<String> columns) {
    columns.addAll(columns);
  }

  Pair<String, List<String>> getTableNameVsColumns() {
    return Pair(tableName, columns);
  }
}
