part of 'zcatalyst_app.dart';

class ZCatalystSDKConfigs {
  ZCatalystSDKConfigs._();

  ZCatalystEnvironment _environment = ZCatalystEnvironment.PRODUCTION;

  ///The Catalyst environment under which the app is currently running.
  ///
  ///Possible Values: [ZCatalystEnvironment.DEVELOPMENT], [ZCatalystEnvironment.PRODUCTION]
  ZCatalystEnvironment get environment => _environment;

  int _projectID = APIConstant.INT_MOCK;

  ///The unique ID of your Catalyst Project.
  int get projectID => _projectID;

  String _accountsPortalBaseURL = APIConstant.STRING_MOCK;

  String get accountsPortalBaseURL => _accountsPortalBaseURL;

  String _apiBaseURL = APIConstant.STRING_MOCK;

  ///The Base URL of the catalyst server through which the internal api is called.
  String get apiBaseURL => _apiBaseURL;

  String _apiVersion = "v1";

  ///The version of the Catalyst internal APIs.
  String get apiVersion => _apiVersion;

  String _oauthScopes = APIConstant.STRING_MOCK;

  ///The scopes that would be used by the SDK to access the Catalyst APIs from your project.
  String get oauthScopes => _oauthScopes;

  String _portalID = APIConstant.STRING_MOCK;

  ///The Unique identifier that maps an application to a project, also known as ZAID.
  String get portalID => _portalID;

  String _redirectURLScheme = APIConstant.STRING_MOCK;

  ///The Redirect URL Scheme of your app that you provided while creating a package for it in the console.
  String get redirectURLScheme => _redirectURLScheme;

  Map<String, String> _requestHeaders = {};

  ///The headers that would be sent by the client in the HTTP requests.
  Map<String, String> get requestHeaders => _requestHeaders;

  bool _printStackTrace = false;

  bool get printStackTrace => _printStackTrace;

  String _clientID = APIConstant.STRING_MOCK;

  ///The Unique identifier of your app client registered in Catalyst.
  String get clientID => _clientID;

  String _clientSecret = APIConstant.STRING_MOCK;

  ///The Secret value generated for a specific clientID, which is passed along with the API hits to ensure that third parties do not access the data.
  String get clientSecret => _clientSecret;

  ///The Client ID required for initializing the SDK for Third Party Authentication.
  String? _jwtClientID;

  String? get jwtClientID => _jwtClientID;

  ///The Client Secret required for initializing the SDK for the Third Party Authentication.
  String? _jwtClientSecret;

  String? get jwtClientSecret => _jwtClientSecret;

  bool _isCustomLoginEnabled = false;

  bool get isCustomLoginEnabled => _isCustomLoginEnabled;

  String? _userAgent;

  String? get userAgent => _userAgent;

  void _initLogger() {
    ZCatalystLogger.initLogger();
    ZCatalystLogger.logInfo("App Configuration Properties: ${toString()}");
  }

  @override
  String toString() {
    _requestHeaders.forEach(
      (key, value) => ZCatalystLogger.logInfo("Header :: $key = $value"),
    );
    return 'Project ID = $projectID, APIVersion = $apiVersion, APIBaseURL = $apiBaseURL, OAuthScopes = $oauthScopes, Client ID = $clientID, Client Secret = $clientSecret, Portal ID = $portalID, Redirect URL = $redirectURLScheme, Is Printing Stack Trace Enabled? = $printStackTrace, JWT Client ID = $jwtClientID, JWT Client Secret = $jwtClientSecret ';
  }
}

class ZCatalystSDKConfigsBuilder {
  final ZCatalystSDKConfigs _configs;

  ZCatalystSDKConfigsBuilder({
    required ZCatalystEnvironment environment,
    required String accountsPortalBaseURL,
    required String apiBaseURL,
    required String clientID,
    required String clientSecret,
    required int projectID,
    required String portalID,
    required String redirectURL,
    required String oauthScopes,
  }) : _configs = ZCatalystSDKConfigs._() {
    _configs._environment = environment;
    _configs._accountsPortalBaseURL = accountsPortalBaseURL;
    _configs._apiBaseURL = apiBaseURL;
    _configs._clientID = clientID;
    _configs._clientSecret = clientSecret;
    _configs._projectID = projectID;
    _configs._portalID = portalID;
    _configs._redirectURLScheme = '$redirectURL://';
    _configs._oauthScopes = oauthScopes;
    addRequestHeaders('PROJECT_ID', portalID);
  }

  ZCatalystSDKConfigsBuilder.customLogin(
      {required ZCatalystEnvironment environment,
      required String accountsPortalBaseURL,
      required String apiBaseURL,
      required String jwtClientID,
      required String jwtClientSecret,
      required int projectID,
      required String portalID,
      required String redirectURL,
      required String oauthScopes})
      : _configs = ZCatalystSDKConfigs._() {
    _configs._environment = environment;
    _configs._accountsPortalBaseURL = accountsPortalBaseURL;
    _configs._apiBaseURL = apiBaseURL;
    _configs._projectID = projectID;
    _configs._jwtClientID = jwtClientID;
    _configs._jwtClientSecret = jwtClientSecret;
    _configs._portalID = portalID;
    _configs._redirectURLScheme = '$redirectURL://';
    _configs._oauthScopes = oauthScopes;
    _configs._isCustomLoginEnabled = true;
    addRequestHeaders('PROJECT_ID', portalID);
  }

  ZCatalystSDKConfigsBuilder setAccountsPortalBaseURL(
      String accountsPortalBaseURL) {
    _configs._accountsPortalBaseURL = accountsPortalBaseURL;
    return this;
  }

  ZCatalystSDKConfigsBuilder addRequestHeaders(String key, String value) {
    _configs._requestHeaders[key] = value;
    return this;
  }

  ZCatalystSDKConfigsBuilder setRequestHeaders(Map<String, String> headers) {
    _configs._requestHeaders = headers;
    return this;
  }

  ZCatalystSDKConfigsBuilder setUserAgent(String userAgent) {
    _configs._userAgent = userAgent;
    return this;
  }

  ZCatalystSDKConfigs build() {
    return _configs;
  }
}

enum ZCatalystEnvironment {
  DEVELOPMENT,
  PRODUCTION;

  @override
  String toString() {
    switch (this) {
      case ZCatalystEnvironment.DEVELOPMENT:
        return 'DEVELOPMENT';

      case ZCatalystEnvironment.PRODUCTION:
        return 'PRODUCTION';
    }
  }
}
