part of '../api/api_request.dart';

enum HttpMethod {
  GET,
  POST,
  PUT,
  DELETE;

  @override
  String toString() {
    switch (this) {
      case HttpMethod.GET:
        return 'GET';
      case HttpMethod.POST:
        return 'POST';
      case HttpMethod.PUT:
        return 'PUT';
      case HttpMethod.DELETE:
        return 'DELETE';
    }
  }
}

enum AggregateFunction {
  SUM,
  AVG,
  COUNT,
  MIN,
  MAX,
  DISTINCT,
}

enum SortOrder {
  ASC,
  DESC,
}

class ZCatalystUtil {
  ///Returns the current time zone of the project.
  ///
  ///On success, The **onSuccess** handler is called with the [APIResponse] object and the **time zone**.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  static Future<void> getCurrentTimeZone(
      Function(APIResponse response, String timeZone) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await APIRequest._(method: HttpMethod.GET, endPoint: 'timezone')
        ._sendRequest(
      onSuccess: (APIResponse response) {
        var responseData = response.responseJson[APIConstant.DATA];
        onSuccess(response, responseData[APIConstant.TIMEZONE]);
      },
      onFailed: (ZCatalystException exception) {
        onFailed(exception);
      },
    );
  }
}

class FileUtil {
  static void validateFileSize(File file, int maxSize) {
    final fileSizeInMb = file.readAsBytesSync().lengthInBytes / (1024 * 1024);
    if (fileSizeInMb > maxSize) {
      throw ZCatalystException(
          code: ErrorCode.FILE_SIZE_EXCEEDED,
          errorMessage: '${ErrorMessage.FILE_SIZE_EXCEEDED} - $maxSize MB');
    }
  }

  static String? getFileNameFromHeaders(Map<String, dynamic> headers) {
    String? fileMeta = headers['content-disposition'];
    String? fileName = fileMeta?.split('=')[1].split('.')[0];
    return fileName;
  }

  static String getFileExtensionFromHeaders(Map<String, dynamic> headers) {
    String contentType = headers['content-type']!;
    String fileExtensionType = contentType.split('/')[1].split(';')[0];
    return fileExtensionType;
  }
}
