import '../api/api_request.dart';

part 'zcql_column.dart';

class ZCatalystQueryBuilder {
  String query = '';

  ZCatalystQueryBuilder select(Set<ZCQLColumn> columns) {
    query += 'SELECT ${getColumnsAsString(columns)} ';
    return this;
  }

  ZCatalystQueryBuilder selectAll() {
    query += 'SELECT * ';
    return this;
  }

  ZCatalystQueryBuilder from(String tableName) {
    query += "FROM $tableName ";
    return this;
  }

  ZCatalystQueryBuilder where(
      String column, Comparator comparator, String value) {
    query += "WHERE $column $comparator $value ";
    return this;
  }

  ZCatalystQueryBuilder and(
      String column, Comparator comparator, String value) {
    query += "AND $column $comparator $value ";
    return this;
  }

  ZCatalystQueryBuilder or(String column, Comparator comparator, String value) {
    query += "OR $column $comparator $value ";
    return this;
  }

  ZCatalystQueryBuilder groupBy(Set<ZCQLColumn> columns) {
    query += "GROUP BY ${getColumnsAsString(columns)} ";
    return this;
  }

  ZCatalystQueryBuilder orderBy(Set<ZCQLColumn> columns, SortOrder sortOrder) {
    query += "ORDER BY ${getColumnsAsString(columns)} $sortOrder";
    return this;
  }

  ZCatalystQueryBuilder innerJoin(String tableName) {
    query += "INNER JOIN $tableName ";
    return this;
  }

  ZCatalystQueryBuilder leftJoin(String tableName) {
    query += "LEFT JOIN $tableName ";
    return this;
  }

  ZCatalystQueryBuilder on(
      String joinColumn1, Comparator comparator, String joinColumn2) {
    query += "ON $joinColumn1 $comparator $joinColumn2 ";
    return this;
  }

  ZCatalystQueryBuilder limit(int offset, int? value) {
    query += "LIMIT $offset ";
    if (value != null) {
      query += ", $this ";
    }
    return this;
  }

  ZCatalystSelectQuery build() {
    return ZCatalystSelectQuery(this);
  }

  String getColumnsAsString(Set<ZCQLColumn> columns) {
    String columnAsString = '';
    for (int index1 = 0; index1 < columns.length; index1++) {
      ZCQLColumn column = columns.elementAt(index1);
      if (column._function != null) {
        Set<String>? listOfColumns = column._names;
        if (listOfColumns != null) {
          for (int index2 = 0; index2 < listOfColumns.length; index2++) {
            if (index2 == 0) {
              columnAsString +=
                  '${column._function} ${listOfColumns.elementAt(index2)}';
            } else {
              columnAsString += listOfColumns.elementAt(index2);
            }
            if (index2 != listOfColumns.length - 1) columnAsString += ', ';
          }
        }
        if (column._name != null) {
          columnAsString += '${column._function} (${column._name})';
        }
        if (index1 != columns.length - 1) columnAsString += ', ';
      } else {
        if (column._name != null) {
          columnAsString += "${column._name}";
        }
        if (index1 != columns.length - 1) {
          columnAsString += ", ";
        }
      }
    }
    return columnAsString;
  }
}

class ZCatalystSelectQuery {
  final String _query;

  String get query => _query;

  ZCatalystSelectQuery(ZCatalystQueryBuilder builder) : _query = builder.query;
}

enum Comparator {
  EQUAL_TO,
  NOT_EQUAL_TO,
  IS,
  IS_NULL,
  IS_NOT_NULL,
  LIKE,
  NOT_LIKE,
  IN,
  NOT_IN,
  GREATER_THAN,
  GREATER_THAN_OR_EQUAL_TO,
  LESSER_THAN,
  LESSER_THAN_OR_EQUAL_TO;

  @override
  String toString() {
    switch (this) {
      case Comparator.EQUAL_TO:
        return '=';
      case Comparator.NOT_EQUAL_TO:
        return '!=';
      case Comparator.IS:
        return 'IS';
      case Comparator.IS_NULL:
        return 'IS NULL';
      case Comparator.IS_NOT_NULL:
        return 'IS NOT NULL';
      case Comparator.LIKE:
        return 'LIKE';
      case Comparator.NOT_LIKE:
        return 'NOT LIKE';
      case Comparator.IN:
        return 'IN';
      case Comparator.NOT_IN:
        return 'NOT IN';
      case Comparator.GREATER_THAN:
        return '>';
      case Comparator.GREATER_THAN_OR_EQUAL_TO:
        return '>=';
      case Comparator.LESSER_THAN:
        return '<';
      case Comparator.LESSER_THAN_OR_EQUAL_TO:
        return '<=';
    }
  }
}
